// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_authentication_bloc/src/domain/data_sources/local/authentication_local_data_source.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

class AuthenticationCacheDataSourceImpl<T extends Object>
    extends AuthenticationLocalDataSource<T> {
  Account? _account;
  T? _data;

  AuthenticationCacheDataSourceImpl();

  @override
  void storeAccount(Account? account) {
    _account = account;
  }

  @override
  void storeData(T? data) {
    _data = data;
  }

  @override
  Account loadAccount() {
    if (_account.isNotNull) {
      return _account!;
    }
    throw ClientException('Cached account is invalid');
  }

  @override
  T loadData() {
    if (_data.isNotNull) {
      return _data!;
    }
    throw ClientException('Cached data is invalid');
  }

  @override
  void destroy() {
    _data = null;
    _account = null;
  }
}
