// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:wyatt_architecture/wyatt_architecture.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account.dart';
import 'package:wyatt_authentication_bloc/src/domain/entities/account_wrapper.dart';
import 'package:wyatt_type_utils/wyatt_type_utils.dart';

abstract class AuthenticationRepository<T> extends BaseRepository {
  FutureResult<Account> signUp({
    required String email,
    required String password,
    
  });

  FutureResult<Account> signInWithEmailAndPassword({
    required String email,
    required String password,
  });

  FutureResult<void> signOut();

  Stream<FutureResult<AccountWrapper<T>>> streamAccount();

  FutureResult<String> getIdentityToken();

  Result<Account, AppException> getAccount();
  Result<Account, AppException> setAccount(Account account);

  Result<T, AppException> getData();
  Result<T, AppException> setData(T data);

  Result<void, AppException> destroyCache();
}
