// Copyright (C) 2022 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:wyatt_authentication_bloc/src/domain/repositories/authentication_repository.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

part 'sign_in_state.dart';

class SignInCubit<Extra> extends Cubit<SignInState> {
  final AuthenticationRepository<Extra> _authenticationRepository;

  final FormValidator _validationStrategy;

  SignInCubit({
    required AuthenticationRepository<Extra> authenticationRepository,
    FormValidator validationStrategy = const EveryInputValidator(),
  })  : _authenticationRepository = authenticationRepository,
        _validationStrategy = validationStrategy,
        super(const SignInState());

  void emailChanged(String value) {
    final Email email = Email.dirty(value);
    emit(
      state.copyWith(
        email: email,
        status: _validationStrategy.rawValidate([email, state.password]),
      ),
    );
  }

  void passwordChanged(String value) {
    final Password password = Password.dirty(value);
    emit(
      state.copyWith(
        password: password,
        status: _validationStrategy.rawValidate([state.email, password]),
      ),
    );
  }

  Future<void> signInWithEmailAndPassword() async {
    if (!state.status.isValidated) {
      return;
    }

    emit(state.copyWith(status: FormStatus.submissionInProgress));

    final uid = await _authenticationRepository.signInWithEmailAndPassword(
      email: state.email.value,
      password: state.password.value,
    );

    emit(
      uid.fold(
        (value) => state.copyWith(status: FormStatus.submissionSuccess),
        (error) => state.copyWith(
          errorMessage: error.message,
          status: FormStatus.submissionFailure,
        ),
      ),
    );
  }
}
